==> comedi-0.5.5.tgz: comedi-0.5.5/module/rti802.c <==
/*
    module/rti802.c
    hardware driver for Analog Devices RTI-802 board

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1999 Anders Blomdell <anders.blomdell@control.lth.se>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <comedi_module.h>


#define RTI802_SIZE 4

#define RTI802_SELECT 0
#define RTI802_DATALOW 1
#define RTI802_DATAHIGH 2

comedi_devinit rti802_init;
static int rti802_rem(comedi_device *dev);

static int rti802_ao(comedi_device *dev, comedi_trig *it)
{
  int result = 0;
  int chan = it->chan;
  if (chan < 0 || chan > 7) {
    result = -EINVAL;
  } else {
    outb(it->chan, dev->iobase+RTI802_SELECT);
    outb(it->data&0xff, dev->iobase + RTI802_DATALOW);
    outb(it->data>>8, dev->iobase + RTI802_DATAHIGH);
  }
  return result;
}

int rti802_init(comedi_device *dev, comedi_devconfig *it)
{
  int i;
  if(strcmp("rti802", it->name) != 0) { return 0; }

  dev->iobase = it->options[0];
  printk("comedi%d: rti802: 0x%04x ", dev->minor, dev->iobase);
  if(check_region(dev->iobase,RTI802_SIZE) < 0){
    printk("I/O port conflict\n");
    return -EIO;
  }
  request_region(dev->iobase, RTI802_SIZE, "rti802");
  dev->iosize = RTI802_SIZE;
  dev->board_name ="rti802";
  dev->driver_name = "rti802";
  dev->chan = 8;
  dev->private = 0;
  dev->chinfo = kmalloc(dev->chan*sizeof(comedi_chinfo), GFP_KERNEL);
  if(!dev->chinfo){ return -ENOMEM; }
  memset(dev->chinfo, 0, dev->chan*sizeof(comedi_chinfo));
  for(i = 0 ; i < 8 ; i++){
    dev->chinfo[i].itrig=rti802_ao;
    dev->chinfo[i].sp=NULL;
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_BITS, COMEDI_RDONLY, 12);
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_MAXDATA, COMEDI_RDONLY, 0xfff);
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_AD, COMEDI_RDONLY, COMEDI_AD_ANALOG);
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_IO, COMEDI_RDONLY, COMEDI_IO_OUTPUT);
  }

  comedi_initbuf(&dev->rdbuf);
  dev->rem = rti802_rem;
  printk("\n");

  return 1;
}

static int rti802_rem(comedi_device *dev)
{
  int i;

  printk("comedi%d: rti802: remove\n", dev->minor);

  for(i = 0; i < dev->chan ; i++) {
    delparamlist(dev->chinfo[i].paramlist);
  }
  kfree(dev->chinfo);
  release_region(dev->iobase, dev->iosize);
  comedi_freebuf(&dev->rdbuf);

  return 0;
}

==> comedi-0.5.6.tgz: comedi-0.5.6/module/rti802.c <==
/*
    module/rti802.c
    hardware driver for Analog Devices RTI-802 board

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1999 Anders Blomdell <anders.blomdell@control.lth.se>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <comedi_module.h>


#define RTI802_SIZE 4

#define RTI802_SELECT 0
#define RTI802_DATALOW 1
#define RTI802_DATAHIGH 2

comedi_devinit rti802_init;
static int rti802_rem(comedi_device *dev);

typedef struct {
  enum { dac_2comp, dac_straight } dac_coding[8];
} rti802_private;
#define devpriv ((rti802_private *)dev->private)

static int rti802_ao(comedi_device *dev, comedi_trig *it)
{
  int result = 0;
  int chan = it->chan;
  if (chan < 0 || chan > 7) {
    result = -EINVAL;
  } else {
    int data = it->data;
    if (devpriv->dac_coding[chan] == dac_2comp) { data -= 0x800; }
    outb(it->chan, dev->iobase+RTI802_SELECT);
    outb(data&0xff, dev->iobase + RTI802_DATALOW);
    outb(data>>8, dev->iobase + RTI802_DATAHIGH);
  }
  return result;
}

int rti802_init(comedi_device *dev, comedi_devconfig *it)
{
  int i;
  int rangetype = 0;
  if(strcmp("rti802", it->name) != 0) { return 0; }

  dev->iobase = it->options[0];
  printk("comedi%d: rti802: 0x%04x ", dev->minor, dev->iobase);
  if(check_region(dev->iobase,RTI802_SIZE) < 0){
    printk("I/O port conflict\n");
    return -EIO;
  }
  request_region(dev->iobase, RTI802_SIZE, "rti802");
  dev->iosize = RTI802_SIZE;
  dev->board_name ="rti802";
  dev->driver_name = "rti802";
  dev->chan = 8;
  dev->private=kmalloc(sizeof(rti802_private)+
                      dev->chan*sizeof(comedi_chinfo),GFP_KERNEL);
  if(!dev->private) { return -ENOMEM; }
  memset(dev->private,0,sizeof(rti802_private)+
        dev->chan*sizeof(comedi_chinfo));
  dev->chinfo = dev->private+sizeof(rti802_private);
  for(i = 0 ; i < 8 ; i++){
    switch (it->options[3+i*2]) {
      case 0:
       devpriv->dac_coding[i] = dac_2comp;
      case 1:
       devpriv->dac_coding[i] = dac_straight;
    }
    dev->chinfo[i].itrig=rti802_ao;
    dev->chinfo[i].sp=NULL;
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_BITS, COMEDI_RDONLY, 12);
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_MAXDATA, COMEDI_RDONLY, 0xfff);
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_AD, COMEDI_RDONLY, COMEDI_AD_ANALOG);
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_IO, COMEDI_RDONLY, COMEDI_IO_OUTPUT);
    switch (it->options[2+i*2]) {
      case 0:
       rangetype = RANGE_rti802_ao_10_bipolar; break;
      case 1:
       rangetype = RANGE_rti802_ao_10_unipolar; break;
    }
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_RANGE_TYPE, COMEDI_RDONLY, rangetype);
    addparam(&dev->chinfo[i].paramlist,
            COMEDI_RANGE, COMEDI_RDONLY, 0);
  }

  comedi_initbuf(&dev->rdbuf);
  dev->rem = rti802_rem;
  printk("\n");

  return 1;
}

static int rti802_rem(comedi_device *dev)
{
  int i;

  printk("comedi%d: rti802: remove\n", dev->minor);

  for(i = 0; i < dev->chan ; i++) {
    delparamlist(dev->chinfo[i].paramlist);
  }
  kfree(dev->chinfo);
  release_region(dev->iobase, dev->iosize);
  comedi_freebuf(&dev->rdbuf);

  return 0;
}
